@@ -1,5 +1,5 @@
 /*
-#  $Id: Driver_xst.h,v 1.3 2002/07/18 14:23:44 timbo Exp $
+#  $Id: Driver_xst.h 2488 2006-02-07 22:24:43Z timbo $
 #  Copyright (c) 2002  Tim Bunce  Ireland
 #
 #  You may distribute under the terms of either the GNU General Public
@@ -9,6 +9,7 @@
 static SV *
 dbixst_bounce_method(char *methname, int params)
 {
+    dTHX;
     /* XXX this 'magic' undoes the dMARK embedded in the dXSARGS of our caller	*/
     /* so that the dXSARGS below can set things up as they were for our caller	*/
     void *xxx = PL_markstack_ptr++;
@@ -16,8 +17,10 @@ dbixst_bounce_method(char *methname, int params)
     int i;
     SV *sv;
     int debug = 0;
+    D_imp_xxh(ST(0));
     if (debug >= 3) {
-	PerlIO_printf(DBILOGFP, "    -> %s (trampoline call with %d (%ld) params)\n", methname, params, (long)items);
+	PerlIO_printf(DBIc_LOGPIO(imp_xxh),
+	    "    -> %s (trampoline call with %d (%ld) params)\n", methname, params, (long)items);
 	xxx = xxx; /* avoid unused var warning */
     }
     EXTEND(SP, params);
@@ -32,7 +35,8 @@ dbixst_bounce_method(char *methname, int params)
     sv = (i) ? POPs : &sv_undef;
     PUTBACK;
     if (debug >= 3)
-	PerlIO_printf(DBILOGFP, "    <- %s= %s (trampoline call return)\n", methname, neatsvpv(sv,0));
+	PerlIO_printf(DBIc_LOGPIO(imp_xxh),
+	    "    <- %s= %s (trampoline call return)\n", methname, neatsvpv(sv,0));
     return sv;
 }
 
@@ -43,14 +47,15 @@ dbdxst_bind_params(SV *sth, imp_sth_t *imp_sth, I32 items, I32 ax)
     /* Handle binding supplied values to placeholders.		*/
     /* items = one greater than the number of params		*/
     /* ax = ax from calling sub, maybe adjusted to match items	*/
+    dTHX;
     int i;
     SV *idx;
     if (items-1 != DBIc_NUM_PARAMS(imp_sth)
 	&& DBIc_NUM_PARAMS(imp_sth) != DBIc_NUM_PARAMS_AT_EXECUTE
     ) {
 	char errmsg[99];
-	sprintf(errmsg,"called with %ld bind variables when %d are needed",
-		items-1, DBIc_NUM_PARAMS(imp_sth));
+	sprintf(errmsg,"called with %d bind variables when %d are needed",
+		(int)items-1, DBIc_NUM_PARAMS(imp_sth));
 	sv_setpv(DBIc_ERRSTR(imp_sth), errmsg);
 	sv_setiv(DBIc_ERR(imp_sth), (IV)-1);
 	return 0;
@@ -72,6 +77,7 @@ dbdxst_bind_params(SV *sth, imp_sth_t *imp_sth, I32 items, I32 ax)
 static SV *
 dbdxst_fetchall_arrayref(SV *sth, SV *slice, SV *batch_row_count)
 {
+    dTHX;
     D_imp_sth(sth);
     SV *rows_rvav;
     if (SvOK(slice)) {  /* should never get here */
@@ -85,6 +91,11 @@ dbdxst_fetchall_arrayref(SV *sth, SV *slice, SV *batch_row_count)
 	IV maxrows = SvOK(batch_row_count) ? SvIV(batch_row_count) : -1;
 	AV *fetched_av;
 	AV *rows_av = newAV();
+	if ( !DBIc_ACTIVE(imp_sth) && maxrows>0 ) {
+	    /* to simplify application logic we return undef without an error	*/
+	    /* if we've fetched all the rows and called with a batch_row_count	*/
+	    return &sv_undef;
+	}
 	av_extend(rows_av, (maxrows>0) ? maxrows : 31);
 	while ( (maxrows < 0 || maxrows-- > 0)
 	    && (fetched_av = dbd_st_fetch(sth, imp_sth))
