@@ -1,6 +1,6 @@
 package DBD::Gofer::Policy::classic;
 
-#   $Id: classic.pm 9391 2007-04-10 15:16:05Z timbo $
+#   $Id: classic.pm 9139 2007-02-19 16:45:56Z timbo $
 #
 #   Copyright (c) 2007, Tim Bunce, Ireland
 #
@@ -10,17 +10,11 @@ package DBD::Gofer::Policy::classic;
 use strict;
 use warnings;
 
-our $VERSION = sprintf("0.%06d", q$Revision: 9391 $ =~ /(\d+)/o);
+our $VERSION = sprintf("0.%06d", q$Revision: 9139 $ =~ /(\d+)/o);
 
 use base qw(DBD::Gofer::Policy::Base);
 
-__PACKAGE__->create_policy_subs({
-
-    # always use connect_cached on server
-    connect_method => 'connect_cached',
-
-    # use same methods on server as is called on client
-    prepare_method => '',
+__PACKAGE__->create_default_policy_subs({
 
     # don't skip the connect check since that also sets dbh attributes
     # although this makes connect more expensive, that's partly offset
@@ -30,19 +24,9 @@ __PACKAGE__->create_policy_subs({
     # most code doesn't rely on sth attributes being set after prepare
     skip_prepare_check => 1,
 
-    # we're happy to use local method if that's the same as the remote
-    skip_default_methods => 1,
-
-    # ping is not important for DBD::Gofer and most transports
+    # ping is almost meaningless for DBD::Gofer and most transports anyway
     skip_ping => 1,
 
-    # only update dbh attributes on first contact with server
-    dbh_attribute_update => 'first',
-
-    # we'd like to set locally_* but can't because drivers differ
-
-    # get_info results usually don't change
-    cache_get_info => 1,
 });
 
 
