@@ -1,6 +1,6 @@
 package DBD::Gofer::Transport::stream;
 
-#   $Id: stream.pm 14623 2010-12-31 12:21:54Z REHSACK $
+#   $Id: stream.pm 9147 2007-02-21 00:54:06Z timbo $
 #
 #   Copyright (c) 2007, Tim Bunce, Ireland
 #
@@ -11,10 +11,11 @@ use strict;
 use warnings;
 
 use Carp;
+use Fcntl;
 
 use base qw(DBD::Gofer::Transport::pipeone);
 
-our $VERSION = sprintf("0.%06d", q$Revision: 14623 $ =~ /(\d+)/o);
+our $VERSION = sprintf("0.%06d", q$Revision: 9147 $ =~ /(\d+)/o);
 
 __PACKAGE__->mk_accessors(qw(
     go_persist
@@ -23,6 +24,9 @@ __PACKAGE__->mk_accessors(qw(
 my $persist_all = 5;
 my %persist;
 
+sub nonblock;
+
+
 
 sub _connection_key {
     my ($self) = @_;
@@ -37,7 +41,7 @@ sub _connection_get {
     $persist = $persist_all if not defined $persist;
     my $key = ($persist) ? $self->_connection_key : '';
     if ($persist{$key} && $self->_connection_check($persist{$key})) {
-        $self->trace_msg("reusing persistent connection $key\n",0) if $self->trace >= 1;
+        DBI->trace_msg("reusing persistent connection $key\n",0) if $self->trace >= 1;
         return $persist{$key};
     }
 
@@ -56,9 +60,7 @@ sub _connection_check {
     my ($self, $connection) = @_;
     $connection ||= $self->connection_info;
     my $pid = $connection->{pid};
-    my $ok = (kill 0, $pid);
-    $self->trace_msg("_connection_check: $ok (pid $$)\n",0) if $self->trace;
-    return $ok;
+    return (kill 0, $pid);
 }
 
 
@@ -66,10 +68,9 @@ sub _connection_kill {
     my ($self) = @_;
     my $connection = $self->connection_info;
     my ($pid, $wfh, $rfh, $efh) = @{$connection}{qw(pid wfh rfh efh)};
-    $self->trace_msg("_connection_kill: closing write handle\n",0) if $self->trace;
     # closing the write file handle should be enough, generally
     close $wfh;
-    # in future we may want to be more aggressive
+    # in code cases in future we may want to be more aggressive
     #close $rfh; close $efh; kill 15, $pid
     # but deleting from the persist cache...
     delete $persist{ $self->_connection_key };
@@ -98,19 +99,19 @@ sub _make_connection {
         unshift @$cmd, qw(ssh -xq), split(' ', $ssh), qw(bash -c), $setup;
     }
 
-    $self->trace_msg("new connection: @$cmd\n",0) if $self->trace;
+    DBI->trace_msg("new connection: @$cmd\n",0) if $self->trace;
 
     # XXX add a handshake - some message from DBI::Gofer::Transport::stream that's
     # sent as soon as it starts that we can wait for to report success - and soak up
     # and report useful warnings etc from ssh before we get it? Increases latency though.
     my $connection = $self->start_pipe_command($cmd);
+    nonblock($connection->{efh});
     return $connection;
 }
 
 
 sub transmit_request_by_transport {
     my ($self, $request) = @_;
-    my $trace = $self->trace;
 
     my $connection = $self->connection_info || do {
         my $con = $self->_connection_get;
@@ -118,94 +119,75 @@ sub transmit_request_by_transport {
         $con;
     };
 
-    my $encoded_request = unpack("H*", $self->freeze_request($request));
-    $encoded_request .= "\015\012";
+    my $frozen_request = unpack("H*", $self->freeze_data($request));
+    $frozen_request .= "\n";
 
     my $wfh = $connection->{wfh};
-    $self->trace_msg(sprintf("transmit_request_by_transport: to fh %s fd%d\n", $wfh, fileno($wfh)),0)
-        if $trace >= 4;
-
     # send frozen request
-    local $\;
-    $wfh->print($encoded_request) # autoflush enabled
+    print $wfh $frozen_request # autoflush enabled
         or do {
-            my $err = $!;
-            # XXX could/should make new connection and retry
+            # XXX should make new connection and retry
             $self->_connection_kill;
-            die "Error sending request: $err";
+            die "Error sending request: $!";
         };
-    $self->trace_msg("Request sent: $encoded_request\n",0) if $trace >= 4;
+    $self->trace_msg("Request: $frozen_request\n",0) if $self->trace >= 4;
 
-    return undef; # indicate no response yet (so caller calls receive_response_by_transport)
+    return;
 }
 
 
 sub receive_response_by_transport {
     my $self = shift;
-    my $trace = $self->trace;
 
-    $self->trace_msg("receive_response_by_transport: awaiting response\n",0) if $trace >= 4;
     my $connection = $self->connection_info || die;
-    my ($pid, $rfh, $efh, $cmd) = @{$connection}{qw(pid rfh efh cmd)};
+    my ($pid, $rfh, $efh) = @{$connection}{qw(pid rfh efh)};
 
-    my $errno = 0;
-    my $encoded_response;
-    my $stderr_msg;
+    # blocks till a newline has been read
+    $! = 0;
+    my $frozen_response = <$rfh>; # always one line
+    my $frozen_response_errno = $!;
 
-    $self->read_response_from_fh( {
-        $efh => {
-            error => sub { warn "error reading response stderr: $!"; $errno||=$!; 1 },
-            eof   => sub { warn "eof reading efh" if $trace >= 4; 1 },
-            read  => sub { $stderr_msg .= $_; 0 },
-        },
-        $rfh => {
-            error => sub { warn "error reading response: $!"; $errno||=$!; 1 },
-            eof   => sub { warn "eof reading rfh" if $trace >= 4; 1 },
-            read  => sub { $encoded_response .= $_; ($encoded_response=~s/\015\012$//) ? 1 : 0 },
-        },
-    });
+    # must read any stderr output _afterwards_
+    # warnings during execution are caught and returned as part
+    # of the response object. So stderr should be silent.
+    my $stderr_msg = do { local $/; <$efh> }; # nonblocking
 
     # if we got no output on stdout at all then the command has
     # probably exited, possibly with an error to stderr.
     # Turn this situation into a reasonably useful DBI error.
-    if (not $encoded_response) {
-        my @msg;
-        push @msg, "error while reading response: $errno" if $errno;
-        if ($stderr_msg) {
-            chomp $stderr_msg;
-            push @msg, sprintf "error reported by \"%s\" (pid %d%s): %s",
-                $self->cmd_as_string,
-                $pid, ((kill 0, $pid) ? "" : ", exited"),
-                $stderr_msg;
-        }
-        die join(", ", "No response received", @msg)."\n";
+    if (not $frozen_response or !chomp $frozen_response) {
+        chomp $stderr_msg if $stderr_msg;
+        my $msg = sprintf("Error reading from %s (pid %d%s): ",
+            $self->cmd_as_string, $pid, (kill 0, $pid) ? "" : ", exited");
+        $msg .= $stderr_msg || $frozen_response_errno;
+        die "$msg\n";
     }
 
-    $self->trace_msg("Response received: $encoded_response\n",0)
-        if $trace >= 4;
+    $self->trace_msg("Response: $frozen_response\n",0)
+        if $self->trace >= 4;
 
     $self->trace_msg("Gofer stream stderr message: $stderr_msg\n",0)
-        if $stderr_msg && $trace;
-
-    my $frozen_response = pack("H*", $encoded_response);
+        if $stderr_msg && $self->trace;
 
     # XXX need to be able to detect and deal with corruption
-    my $response = $self->thaw_response($frozen_response);
+    my $response = $self->thaw_data(pack("H*",$frozen_response));
 
-    if ($stderr_msg) {
-        # add stderr messages as warnings (for PrintWarn)
-        $response->add_err(0, $stderr_msg, undef, $trace)
-            # but ignore warning from old version of blib
-            unless $stderr_msg =~ /^Using .*blib/ && "@$cmd" =~ /-Mblib/;
-    }   
+    # add any stderr messages as a warning (ie PrintWarn)
+    $response->add_err(0, $stderr_msg, undef, $self->trace)
+        if $stderr_msg;
 
     return $response;
 }
 
-sub transport_timedout {
-    my $self = shift;
-    $self->_connection_kill;
-    return $self->SUPER::transport_timedout(@_);
+
+
+# nonblock($fh) puts filehandle into nonblocking mode
+sub nonblock {
+  my $fh = shift;
+  my $flags = fcntl($fh, F_GETFL, 0)
+        or croak "Can't get flags for filehandle $fh: $!";
+  fcntl($fh, F_SETFL, $flags | O_NONBLOCK)
+        or croak "Can't make filehandle $fh nonblocking: $!";
 }
 
 1;
@@ -213,7 +195,7 @@ sub transport_timedout {
 __END__
 
 =head1 NAME
-
+    
 DBD::Gofer::Transport::stream - DBD::Gofer transport for stdio streaming
 
 =head1 SYNOPSIS
@@ -272,21 +254,17 @@ Automatically reconnect (within reason) if there's a transport error.
 
 Decide on default for persistent connection - on or off? limits? ttl?
 
-=head1 AUTHOR
+=head1 AUTHOR AND COPYRIGHT
 
-Tim Bunce, L<http://www.tim.bunce.name>
+The DBD::Gofer, DBD::Gofer::* and DBI::Gofer::* modules are
+Copyright (c) 2007 Tim Bunce. Ireland.  All rights reserved.
 
-=head1 LICENCE AND COPYRIGHT
-
-Copyright (c) 2007, Tim Bunce, Ireland. All rights reserved.
-
-This module is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself. See L<perlartistic>.
+You may distribute under the terms of either the GNU General Public License or
+the Artistic License, as specified in the Perl README file.
 
 =head1 SEE ALSO
 
-L<DBD::Gofer::Transport::Base>
-
 L<DBD::Gofer>
 
 =cut
+
